// SPDX-FileCopyrightText: 2022 Matteo Settenvini <matteo.settenvini@montecristosoftware.eu>
// SPDX-License-Identifier: AGPL-3.0-or-later

use {
    crate::api_client::ApiClient,
    crate::recipe::{Ingredient, Recipe},
    anyhow::Result,
    chrono::{Duration, Local},
    icalendar::Component,
    regex::Regex,
    std::collections::HashSet,
    std::ops::Range,
};

pub async fn with(api_client: &ApiClient, calendar_name: &str, days: u32) -> Result<()> {
    let ids = map_events_to_recipe_ids(api_client, calendar_name, days).await?;
    let ingredients = get_ingredients(api_client, ids).await?;

    todo!("Recipe ingredients: {:?}", ingredients)

    // let ingredients = merge_ingredients(ingredients);
    // let md = prepare_grocery_list(&ingredients);
    // save_grocery_list(api_client, "", md).await?;
}

async fn map_events_to_recipe_ids(
    api_client: &ApiClient,
    calendar_name: &str,
    days: u32,
) -> Result<HashSet<usize>> {
    let date_range = Range {
        start: Local::now(),
        end: Local::now() + Duration::days(days as i64),
    };

    let all_events = api_client.get_events(calendar_name, date_range).await?;

    let recipe_id_regex: Regex = Regex::new(r"cookbook@(\d+)").unwrap();
    let recipe_ids = all_events
        .iter()
        .flat_map(|event| event.property_value("DESCRIPTION"))
        .flat_map(|descr| recipe_id_regex.captures(descr))
        .flat_map(|c| c.get(1))
        .flat_map(|m| m.as_str().parse::<usize>())
        .collect::<HashSet<_>>();

    Ok(recipe_ids)
}

async fn get_ingredients<RecipeIds>(
    api_client: &ApiClient,
    recipe_ids: RecipeIds,
) -> Result<Vec<Ingredient>>
where
    RecipeIds: IntoIterator<Item = usize>,
{
    let ingredients = recipe_ids.into_iter().map(|id: usize| async move {
        // TODO code duplicated with schedule_csv::get_all_recipes
        let recipe_url = format!("apps/cookbook/api/recipes/{id}");
        let response = api_client
            .rest()
            .get(api_client.base_url().join(&recipe_url).unwrap())
            .send()
            .await
            .expect(&format!("Cannot fetch recipe with id {}", id));

        response.json::<Recipe>().await.map(|r| r.ingredients)
    });

    let ingredients = futures::future::try_join_all(ingredients).await?;
    Ok(ingredients.into_iter().flatten().collect())
}
